#include "UDPprotect.h"

#define max_comps	100		// maximum number of computers

static UINT udp_port=4444;
static UINT send_delay=1000;

static void ErrorOUT(char *s)
{
	printf("ERROR: %s (CODE=%d)\n",s,WSAGetLastError());
}


static SOCKET SockBind()
{
	SOCKET s=socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
	if (s==INVALID_SOCKET) printf("socket()\n");

	sockaddr_in sin;
	sin.sin_family=AF_INET;
	sin.sin_port=htons(udp_port);
	sin.sin_addr.S_un.S_addr=INADDR_ANY;

	int fromlen=sizeof(sin);

	int one=1;
	setsockopt(s,SOL_SOCKET,SO_REUSEADDR,(char *)&one,sizeof(one));
	
	int suc=bind(s,(sockaddr *)&sin,fromlen);
	if (suc!=0) ErrorOUT("bind()");

	return s;
}

static void RecvUDP(SOCKET s,char *compname)
{
	if (recvfrom(s,compname,MAX_COMPUTERNAME_LENGTH,0,NULL,NULL)==SOCKET_ERROR)
		ErrorOUT("recvfrom()");
}


static void SendUDP()
{
	SOCKET s=socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
	if (s==INVALID_SOCKET) printf("socket()\n");

	char compname[MAX_COMPUTERNAME_LENGTH+1];
	gethostname(compname,MAX_COMPUTERNAME_LENGTH);
	hostent *hp=gethostbyname(compname);
	hp->h_addr[3]=(BYTE)255;

	sockaddr_in sin;
	sin.sin_family=AF_INET;
	sin.sin_port=htons(udp_port);
	memcpy(&(sin.sin_addr.S_un.S_addr),hp->h_addr,hp->h_length);

	int ttl=7;	// Arbitrary TTL value.
	setsockopt(s,IPPROTO_IP,IP_MULTICAST_TTL,(char *)&ttl,sizeof(ttl));
	setsockopt(s,IPPROTO_IP,IP_MULTICAST_IF,(char *)&(sin.sin_addr.S_un.S_addr),4);

	int suc=sendto(s,compname,MAX_COMPUTERNAME_LENGTH,0,(sockaddr *)&sin,sizeof(sin));
	if (suc==SOCKET_ERROR) ErrorOUT("sendto()");
}

static void SendNetwork()
{
	for (;; Sleep(send_delay)) SendUDP();
}


void UDPprotect(UINT port,UINT delay,void (*action)(UINT,char *))
{
	udp_port=port;		// UDP port
	send_delay=delay;	// delay between sendings (ms)

	DWORD id;
	HANDLE htr=CreateThread(0,0,(LPTHREAD_START_ROUTINE)SendNetwork,0,THREAD_SET_INFORMATION,&id);
 	SetThreadPriority(htr,THREAD_PRIORITY_LOWEST);


	char *comps[max_comps];
	UINT ncomps=0;

	SOCKET s=SockBind();
	for (;;) {
		char compname[MAX_COMPUTERNAME_LENGTH+1];
		RecvUDP(s,compname);

		BOOL found=FALSE;
		for (UINT i=0; i<ncomps; i++) {
			if (!stricmp(comps[i],compname)) {
				found=TRUE; break;
			}
		}
		if (found==FALSE) {
			if (ncomps+1<max_comps) {
				comps[ncomps]=new char[MAX_COMPUTERNAME_LENGTH+1];
				strcpy(comps[ncomps],compname);
			} 
			ncomps++;
			if (action!=NULL) action(ncomps,compname);
		}
	}
}